@@ -29,7 +29,7 @@ preconfigure_target () {
 	target_configure_initial_keymap_font || failed+=('keymap/font setting')
 	target_configure_fstab || failed+=('fstab configuration')
 	target_configure_network || failed+=('network config export')
-	target_configure_mirrorlist || failed+=('mirrorlist configuration')
+	configure_mirrorlist target || failed+=('mirrorlist configuration')
 	target_configure_time || failed+=('time configuration')
 	target_configure_initcpio || failed+=('initcpio configuration')
 	# TODO: we should probably update /etc/crypttab if the user has non-/ encrypted disks.
@@ -271,8 +271,9 @@ interactive_autoprepare()
 		DISC=${BLOCKFRIENDLY[0]}
 	fi
 
-	FSOPTS=()
-	for fs in ext2 ext3 ext4 reiserfs xfs jfs vfat nilfs2
+	local FSOPTS=()
+	local fs
+	for fs in ext2 ext3 ext4 reiserfs xfs jfs vfat nilfs2 btrfs
 	do
 		check_is_in $fs "${possible_fs[@]}" && FSOPTS+=($fs "${filesystem_names[$fs]}")
 	done
@@ -385,6 +386,7 @@ interactive_filesystem ()
 	local fs_opts=
 	local fs_label=
 	local fs_params=
+	local default
 	NEW_FILESYSTEM=
 	if [ -n "$fs_string" ]
 	then
@@ -410,7 +412,8 @@ interactive_filesystem ()
 	if [ "$NEW_FILESYSTEM" != no_fs ]
 	then
 		# Determine which filesystems/blockdevices are possible for this blockdevice
-		FSOPTS=()
+		local FSOPTS=()
+		local fs
 		for fs in ${fs_on[$part_type]}
 		do
 			check_is_in $fs "${possible_fs[@]}" && FSOPTS+=($fs "${filesystem_names[$fs]}")
@@ -425,8 +428,7 @@ interactive_filesystem ()
 			notify "Automatically picked the ${FSOPTS[1]} filesystem.  It's the only option for $part_type blockdevices"
 			fs_type=${FSOPTS[0]}
 		else
-			default=no
-			[ -n "$fs_type" ] && default="$fs_type"
+			default=${fs_type:-no}
 			extratext="Select a filesystem for $part:"
 			[ "$fs_create" == no ] && extratext="Select which filesystem $part is.  Make sure you get this right" #otherwise he'll be screwed when we try to mount it :)
 			ask_option $default "Select filesystem" "$extratext" required "${FSOPTS[@]}" || return 1
@@ -436,8 +438,7 @@ interactive_filesystem ()
 		# ask mountpoint, if relevant
 		if check_is_in $fs_type "${fs_mountable[@]}"
 		then
-			default=no
-			[ -n "$fs_mountpoint" ] && default="$fs_mountpoint"
+			default=${fs_mountpoint:-no}
 			ask_option $default "Select the mountpoint" "Select a mountpoint for $part" required / 'root' /boot 'files for booting' /home 'home directories' /var 'variable files' /tmp 'temporary files' custom 'enter a custom mountpoint' || return 1
 			fs_mountpoint=$ANSWER_OPTION
 			[ "$default" == 'no' ] && default=
@@ -466,13 +467,14 @@ interactive_filesystem ()
 		if [ "$fs_create" == yes ] && [ "$fs_type" = lvm-vg ]
 		then
 			# add $part to $fs_params if it's not in there because the user wants this enabled by default. TODO: we should find something out so you can't disable $part. (would be weird to have a vg listed on $part and not have $part it fs_params)
-			pv=${part/+/}
+			local pv=${part/+/}
 			if ! egrep -q "$pv(\$| )" <<< "$fs_params"; then
 				[ -n "$fs_params" ] && fs_params="$fs_params "
 				fs_params="$fs_params$pv"
 			fi
 
-			list=()
+			local list=()
+			local pv
 			for pv in $fs_params
 			do
 				list+=("$pv" ^ ON)
@@ -554,12 +556,13 @@ remove_blockdevice ()
 	local part_type=$2  # a part should always have a type
 	local part_label=$3 # must be given
 
-	target="$part $part_type $part_label"
+	local target="$part $part_type $part_label"
 	declare target_escaped=${target//\//\\/} # note: apparently no need to escape the '+' sign for sed.
 	declare target_escawk=${target_escaped/+/\\+} # ...but that doesn't count for awk
 	fs_string=`awk "/^$target_escawk / { print \$4}" $TMP_BLOCKDEVICES` #TODO: fs_string is the entire line, incl part?
 	debug 'UI-INTERACTIVE' "Cleaning up partition $part (type $part_type, label $part_label).  It has the following FS's on it: $fs_string"
 	sed -i "/$target_escaped/d" $TMP_BLOCKDEVICES || show_warning "blockdevice removal" "Could not remove partition $part (type $part_type, label $part_label).  This is a bug. please report it"
+	local fs
 	for fs in `sed 's/|/ /g' <<< $fs_string`
 	do
 		fs_type=`       cut -d ';' -f 1 <<< $fs`
@@ -573,7 +576,7 @@ remove_blockdevice ()
 
 
 interactive_filesystems() {
-
+	local PART_ACCESS
 	if [ ! -f $TMP_BLOCKDEVICES ] || ! ask_yesno "Previous blockdevice definitions found:\n`cat $TMP_BLOCKDEVICES`\n\
 		Use these as a starting point?  Make sure your disk(s) are partitioned correctly so your definitions can be applied. Pick 'no' when in doubt to start from scratch" no
 	then
@@ -586,7 +589,7 @@ interactive_filesystems() {
 		"label"	"by Disk-Label (Will use the filesystem labels where you give them, and fall back on 'dev' otherwise)" \
 		"uuid"	"by Universally Unique Identifier (You don't need to do anything, but doesn't look pretty)" || return 1
 	PART_ACCESS=$ANSWER_OPTION
-	ALLOK=0
+	local ALLOK=0
 	while [ "$ALLOK" = 0 ]
 	do
 		# Let the user make filesystems and mountpoints. USERHAPPY becomes 1 when the user hits DONE.
@@ -595,23 +598,24 @@ interactive_filesystems() {
 		while [ "$USERHAPPY" = 0 ]
 		do
 			# generate a menu based on the information in the datafile
-			menu_list=()
+			local menu_list=()
+			local part, type, label, fs
 			while read part type label fs
 			do
 				parse_filesystem_string "$fs" '' '-'
-				fs_create_display=N
+				local fs_create_display=N
 				[ "$fs_create" = yes ] && fs_create_display=Y
-				fs_display="$fs_type $fs_create_display $fs_mountpoint $fs_opts $fs_label $fs_params"
+				local fs_display="$fs_type $fs_create_display $fs_mountpoint $fs_opts $fs_label $fs_params"
 
-				part_label_display=-
-				part_size_display=-
+				local part_label_display=-
+				local part_size_display=-
 				[ "$label" != no_label ] && part_label_display="$label"
 				# add size in MiB for existing blockdevices (eg not for mapper devices that are not yet created yet)
 				if [ -b "${part/+/}" ] && get_blockdevice_size ${part/+/} MiB # test -b <-- exit's 0, test -b '' exits >0.
 				then
 					part_size_display="${BLOCKDEVICE_SIZE}MiB"
 				fi
-				part_display="$part $type $part_label_display $part_size_display"
+				local part_display="$part $type $part_label_display $part_size_display"
 				menu_list+=("$part_display" "$fs_display")
 			done < $TMP_BLOCKDEVICES
 
@@ -624,12 +628,16 @@ device type label size type create? mountpoint options label params" required "$
 
 			part=$(echo $ANSWER_OPTION | cut -d ' ' -f1)
 			getpartinfo $part ''
+			local fs=$PARTINFO_fs
+			local part_label=$PARTINFO_part_label
+			local part_type=$PARTINFO_part_type
 
 			if [ $part_type = lvm-vg ] # one lvm VG can host multiple LV's so that's a bit a special blockdevice...
 			then
-				list=()
+				local list=()
 				if [ -n "$fs" ]
 				then
+					local lv
 					for lv in `sed 's/|/ /g' <<< $fs`
 					do
 						label=$(cut -d ';' -f 6 <<< $lv)
@@ -639,7 +647,7 @@ device type label size type create? mountpoint options label params" required "$
 				fi
 				list+=(empty NEW)
 				ask_option empty "Manage LV's on this VG" "Edit/create new LV's on this VG:" required "${list[@]}" && {
-					EDIT_VG=$ANSWER_OPTION
+					local EDIT_VG=$ANSWER_OPTION
 					if [ "$ANSWER_OPTION" = empty  ]
 					then
 						# a new LV must be created on this VG
@@ -653,13 +661,14 @@ device type label size type create? mountpoint options label params" required "$
 						fi
 					else
 						# an existing LV will be edited and it's settings updated
+						local lv
 						for lv in `sed 's/|/ /g' <<< $fs`
 						do
 							label=$(cut -d ';' -f 6 <<< $lv)
 							[ "$label" = "$EDIT_VG" ] && found_lv="$lv"
 						done
 						interactive_filesystem $part $part_type $part_label "$found_lv"
-						newfs=
+						local newfs, add
 						for lv in `sed 's/|/ /g' <<< $fs`
 						do
 							label=$(cut -d ';' -f 6 <<< $lv)
@@ -690,8 +699,8 @@ device type label size type create? mountpoint options label params" required "$
 		done
 
 		# Check all conditions that need to be fixed and ask the user if he wants to go back and correct them
-		errors=
-		warnings=
+		local errors=
+		local warnings=
 
 		grep -q ';/boot;' $TMP_BLOCKDEVICES || warnings="$warnings\n-No separate /boot filesystem"
 		grep -q ';/;'     $TMP_BLOCKDEVICES || errors="$errors\n-No filesystem with mountpoint /"
@@ -699,7 +708,7 @@ device type label size type create? mountpoint options label params" required "$
 
 		if [ -n "$errors$warnings" ]
 		then
-			str="The following issues have been detected:\n"
+			local str="The following issues have been detected:\n"
 			[ -n "$errors" ] && str="$str\n - Errors: $errors"
 			[ -n "$warnings" ] && str="$str\n - Warnings: $warnings"
 			[ -n "$errors" ] && str="$str\nIt is highly recommended you go back to fix at least the errors."
@@ -736,7 +745,8 @@ interactive_select_packages() {
 
 	repos=`list_pacman_repos target`
 	notify "Package selection is split into three stages. First, you will select a bootloader. Then, you will select package groups that contain packages that you may be interested in. Lastly, you will be presented with a full list of packages for each group, allowing you to fine-tune.\n\n
-Note that right now the packages (and groups) selection is limited to the repos available at this time ($repos).  Once you have your Arch system up and running, you have access to more repositories and packages.\n\n
+Note that packages (and groups) selection are limited to the repos you enabled earlier ($repos)\n
+Once you have your Arch system up and running, you have access to more repositories and packages.\n\n
 If any previous configuration you've done until now (like fancy filesystems) require extra packages, we've already preselected them for your convenience"
 
 	# show group listing for group selection, base is ON by default, all others are OFF
@@ -757,8 +767,8 @@ If any previous configuration you've done until now (like fancy filesystems) req
 	ask_checklist "Select Package groups\nDo not deselect base unless you know what you're doing!" 0 "${grouplist[@]}" || return 1
 	grouplist=("${ANSWER_CHECKLIST[@]}")
 
-	# get sorted array of available packages, with their groups. TODO: we should use $repos here
-	local pkgall=($(list_packages repo core | cut -d ' ' -f2))
+	# get sorted array of available packages, with their groups.
+	local pkgall=($(list_packages repo "$repos" | cut -d ' ' -f2 | sort | uniq)) # list of all packages
 	pkginfo "${pkgall[@]}"
 
 	# build the list of options, sorted primarily by group, then by packagename (this is already). marking where appropriate
@@ -770,7 +780,7 @@ If any previous configuration you've done until now (like fancy filesystems) req
 			mark=ON
 		fi
 		pkglist+=("$pkgname" "$pkggroup" $mark "$pkgname $pkgver: $pkgdesc")
-	done < <(echo "$PACKAGE_INFO" | sort -f -k 3)
+	done < <(echo "$PACKAGE_INFO" | sort -f -k 3b,3 -k 1,1)
 
 	[ ${#pkglist[@]} -eq 0 ] && show_warning "No packages found" "Sorry. I could not find any packages. maybe your network is not setup correctly, you lost connection, no mirror setup, bad group, ..." && return 1
 
@@ -785,6 +795,7 @@ If any previous configuration you've done until now (like fancy filesystems) req
 # args: none
 # returns: 1 on failure
 interactive_runtime_network() {
+	notify "If you wish to load your ethernet modules manually, please do so now in an another terminal."
 	local ifaces
 	ifaces=$(ifconfig -a |grep "Link encap:Ethernet"|sed 's/ \+Link encap:Ethernet \+HWaddr \+/ /g')
 
@@ -1220,16 +1231,16 @@ generate_syslinux_menu () {
 # (0) Arch Linux
 LABEL arch
     MENU LABEL Arch Linux
-    LINUX ../vmlinuz26
+    LINUX ../vmlinuz-linux
     APPEND $kernel_parameters
-    INITRD ../kernel26.img
+    INITRD ../initramfs-linux.img
 
 # (1) Arch Linux Fallback
 LABEL archfallback
     MENU LABEL Arch Linux Fallback
-    LINUX ../vmlinuz26
+    LINUX ../vmlinuz-linux
     APPEND $kernel_parameters
-    INITRD ../kernel26-fallback.img
+    INITRD ../initramfs-linux-fallback.img
 
 # (2) Windows
 #LABEL windows
@@ -1273,10 +1284,12 @@ get_kernel_parameters() {
 
 	case "$PART_ACCESS" in
 		label)
-			local label="$(getlabel $rootpart)" && \
+			local label
+			label="$(getlabel $rootpart)" && \
 			rootpart="/dev/disk/by-label/$label" ;;
 		uuid)
-			local uuid="$(getuuid $rootpart)" && \
+			local uuid
+			uuid="$(getuuid $rootpart)" && \
 			rootpart="/dev/disk/by-uuid/$uuid" ;;
 	esac
 
@@ -1322,41 +1335,69 @@ get_kernel_parameters() {
 
 
 # displays installation source selection menu
-# and sets up relevant config files
+# and sets up relevant configuration
 #
 # params: none
 # returns: nothing
 interactive_select_source() {
-	var_PKG_SOURCE_TYPE=
-	var_FILE_URL="file:///src/core/pkg"
-	var_SYNC_URL=
-
-	ask_option no "Source selection" "Please select an installation source" required \
-	"cd"  "CD-ROM or OTHER SOURCE" \
-	"net" "NET (FTP/HTTP)" || return 1
-
-	case $ANSWER_OPTION in
-		"cd") var_PKG_SOURCE_TYPE="cd" ;;
-		"net") var_PKG_SOURCE_TYPE="net" ;;
-	esac
-
-	if [ "$var_PKG_SOURCE_TYPE" = "cd" ]; then
-		TITLE="Arch Linux CDROM or OTHER SOURCE Installation"
-		notify "Packages included on this disk have been mounted to /src/core/pkg. If you wish to use your own packages from another source, manually mount them there."
-		if [ ! -d /src/core/pkg ]; then
-			notify "Package directory /src/core/pkg is missing!"
-			return 1
+	shopt -s nullglob
+	repos_onboard=(/repo/*)
+	shopt -u nullglob
+	list=()
+	for repo in "${repos_onboard[@]##*/}"; do
+		if [ $repo == 'core' ]; then
+			list+=($repo-local "$repo mounted in install medium" ON)
+		else
+			list+=($repo-local "$repo mounted in install medium" OFF)
 		fi
-		echo "Using CDROM for package installation" >$LOG
-	else
-		TITLE="Arch Linux NET (FTP/HTTP) Installation"
-		notify "If you wish to load your ethernet modules manually, please do so now in an another terminal."
+	done
+	for repo in $(list_possible_repos); do
+		if [ $repo == 'core' -a ! -d /repo/core ]; then
+			list+=($repo-remote "$repo on remote mirror" ON)
+		else
+			list+=($repo-remote "$repo on remote mirror" OFF)
+		fi
+	done
+	# TODO: enable repositories in pacman.conf on target system based on this
+	warn="Be sure to select [core], that's the only must."
+	[ -d /repo/core ] && warn="Be sure to select at least one [core] location,
+	having [core] available is the only must.  DON'T combine a local repository
+	with remote mirrors unless you know what you're doing (this will cause BROKEN packages)!"
+	ask_checklist "Please select which repositories you want to activate.  $warn.
+	Note that this does not affect pacman.conf on the target system, you can
+	finetune repositories later in the \"Configure system\" step.
+	If you want to manually mount other repositories first, you can mount them under /repo and rerun this step." 0 "${list[@]}" || return 1
+	repos="${ANSWER_CHECKLIST[@]}"
+	if echo ${repos[@]} | grep -q remote; then
+		interactive_select_mirror
 	fi
+	if echo ${repos[@]} | grep -q local && echo ${repos[@]} | grep -q remote; then
+		ask_option local-first "Repository ordering" "How do you want to order
+		them?  Like mentioned before, you are aware this will usually BREAK your system right?" '' \
+		"local_first" "Only consult remote when package is not found locally. Consumes less bandwith" \
+		"remote_first" "Consult local repository only when package is not found on remote.  Can yield more up-to-date system"
+		repo_ordering=$ANSWER_OPTION
+		if [ $repo_ordering = 'remote_first' ]; then
+			# we use the property that the reverse of remote ('etomer') sorts
+			# before the reverse of local
+			repos=($(echo ${repos[@]} | tr ' ' '\n' | rev | sort | rev | tr '\n' ' '))
+		fi
+	fi
+	TARGET_REPOSITORIES=()
+	for repo in ${repos[@]}; do
+		if echo $repo | grep -q local; then
+			repo=$(echo $repo | sed 's/-local//')
+			TARGET_REPOSITORIES+=($repo 'file:///repo/$repo/$arch')
+		else
+			repo=$(echo $repo | sed 's/-remote//')
+			TARGET_REPOSITORIES+=($repo $var_MIRRORLIST)
+		fi
+	done
 	return 0
 }
 
 
-# Prompt user for preferred mirror and set $var_SYNC_URL
+# Prompt user for preferred mirror and set $MIRROR
 # args: none
 # returns: nothing
 interactive_select_mirror() { 
@@ -1367,12 +1408,12 @@ interactive_select_mirror() {
 	local _server=$ANSWER_OPTION
 	if [ "${_server}" = "Custom" ]; then
 		ask_string "Enter the full URL to core repo." "ftp://ftp.archlinux.org/core/os/$var_ARCH" || return 1
-		var_SYNC_URL=${ANSWER_STRING/\/core\///\$repo/} #replace '/core/' by '/$repo/'
+		MIRROR=${ANSWER_STRING/\/core\///\$repo/} #replace '/core/' by '/$repo/'
 	else
 		# Form the full URL for our mirror by grepping for the server name in
 		# our mirrorlist and pulling the full URL out.
 		# Ensure that if it was listed twice we only return one line for the mirror.
-		var_SYNC_URL=$(egrep -o "${_server}.*" "${var_MIRRORLIST}" | head -n1)
+		MIRROR=$(egrep -o "${_server}.*" "${var_MIRRORLIST}" | head -n1)
 	fi
-	echo "Using mirror: $var_SYNC_URL" >$LOG
+	echo "Using mirror: $MIRROR" >$LOG
 }
